/*   FILE: PrefWindow.java
 *   DATE OF CREATION:   Thu Jan 09 15:47:07 2003
 *   Copyright (c) Emmanuel Pietriga, 2002. All Rights Reserved
 *   Copyright (c) INRIA, 2004-2010. All Rights Reserved
 *   Licensed under the GNU LGPL. For full terms see the file COPYING.
 *   $Id$
 */

package net.claribole.zgrviewer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import fr.inria.zvtm.glyphs.*;

class PrefWindow extends JFrame implements ActionListener, MouseListener {

    ZGRViewer application;
    GraphicsManager grMngr;

    JTabbedPane tabbedPane;

    JButton okPrefs, savePrefs;

    //Misc prefs
    JCheckBox saveWindowLayoutCb, dynaspotCb;
    JCheckBox antialiascb, silentCb;
    JTextField cmdLOptsTf;
    JCheckBox sdZoomCb;
    JSlider sdZoomSlider;
    JSpinner mFactorSpinner;
    ColorIndicator highlightColor;

    //directory panel
    JButton browseTmpDirBt, browseGraphDirBt, browseNeatoBt, browseCircoBt, browseTwopiBt, browseDotBt,
        browseFontDirBt;
    JTextField tmpDirTF, graphDirTF, neatoPathTF, circoPathTF, twopiPathTF, dotPathTF, fontDirTF;
    JCheckBox cb1;

    //web browser panel
    JRadioButton detectBrowserBt, specifyBrowserBt;
    JTextField browserPathTf, browserOptsTf;
    JButton brw6, webHelpBt;
    JLabel pathLb, optLb;

    //proxy/firewall
    JCheckBox useProxyCb;
    JLabel proxyHostLb, proxyPortLb;
    JTextField proxyHostTf, proxyPortTf;
    JButton proxyHelpBt;

    PrefWindow(final ZGRViewer app, final GraphicsManager gm) {
        this.application = app;
        this.grMngr = gm;
        this.tabbedPane = new JTabbedPane();

        //misc panel
        final JPanel miscPane = new JPanel();
        final GridBagLayout gridBag0 = new GridBagLayout();
        final GridBagConstraints constraints0 = new GridBagConstraints();
        constraints0.fill = GridBagConstraints.HORIZONTAL;
        constraints0.anchor = GridBagConstraints.WEST;
        miscPane.setLayout(gridBag0);

        //save window layout checkbox
        this.saveWindowLayoutCb =
            new JCheckBox("Save/Restore Window Layout at Startup", ConfigManager.SAVE_WINDOW_LAYOUT);
        Utils.buildConstraints(constraints0, 0, 0, 2, 1, 100, 10);
        gridBag0.setConstraints(this.saveWindowLayoutCb, constraints0);
        miscPane.add(this.saveWindowLayoutCb);
        // activate dynaspot pointing 
        this.dynaspotCb = new JCheckBox("Activate DynaSpot pointing", ConfigManager.DYNASPOT);
        this.dynaspotCb.addActionListener(this);
        Utils.buildConstraints(constraints0, 0, 1, 2, 1, 100, 10);
        gridBag0.setConstraints(this.dynaspotCb, constraints0);
        miscPane.add(this.dynaspotCb);
        //antialiasing
        this.antialiascb = new JCheckBox("Antialiasing", ConfigManager.ANTIALIASING);
        this.antialiascb.addActionListener(this);
        Utils.buildConstraints(constraints0, 0, 2, 2, 1, 100, 10);
        gridBag0.setConstraints(this.antialiascb, constraints0);
        miscPane.add(this.antialiascb);
        // -q option
        this.silentCb =
            new JCheckBox("GraphViz programs should not issue warnings (v1.10 and above)", ConfigManager.FORCE_SILENT);
        this.silentCb.addActionListener(this);
        Utils.buildConstraints(constraints0, 0, 3, 2, 1, 100, 10);
        gridBag0.setConstraints(this.silentCb, constraints0);
        miscPane.add(this.silentCb);
        //command line options
        final JLabel cmdLOptsLb = new JLabel("dot/neato command line options (-T will be ignored)");
        Utils.buildConstraints(constraints0, 0, 4, 2, 1, 100, 10);
        gridBag0.setConstraints(cmdLOptsLb, constraints0);
        miscPane.add(cmdLOptsLb);
        this.cmdLOptsTf = new JTextField(ConfigManager.CMD_LINE_OPTS);
        Utils.buildConstraints(constraints0, 0, 5, 2, 1, 100, 10);
        gridBag0.setConstraints(this.cmdLOptsTf, constraints0);
        miscPane.add(this.cmdLOptsTf);
        this.sdZoomCb = new JCheckBox("Enable speed-dependent automatic zooming");
        Utils.buildConstraints(constraints0, 0, 6, 2, 1, 100, 10);
        gridBag0.setConstraints(this.sdZoomCb, constraints0);
        miscPane.add(this.sdZoomCb);
        this.sdZoomCb.setSelected(ZGRViewer.cfgMngr.isSDZoomEnabled());
        final ActionListener a31 = new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                final boolean b = PrefWindow.this.sdZoomCb.isSelected();
                ZGRViewer.cfgMngr.setSDZoomEnabled(b);
                PrefWindow.this.sdZoomSlider.setEnabled(b);
            }
        };
        this.sdZoomCb.addActionListener(a31);
        this.sdZoomSlider = new JSlider(2, 10, ZGRViewer.cfgMngr.getSDZoomFactor());
        this.sdZoomSlider.setLabelTable(this.sdZoomSlider.createStandardLabels(1));
        this.sdZoomSlider.setPaintLabels(true);
        this.sdZoomSlider.setPaintTicks(true);
        this.sdZoomSlider.setSnapToTicks(true);
        this.sdZoomSlider.setPaintTrack(true);
        this.sdZoomSlider.setEnabled(ZGRViewer.cfgMngr.isSDZoomEnabled());
        Utils.buildConstraints(constraints0, 0, 7, 2, 1, 100, 10);
        gridBag0.setConstraints(this.sdZoomSlider, constraints0);
        miscPane.add(this.sdZoomSlider);
        final ChangeListener cl0 = new ChangeListener() {
            @Override
            public void stateChanged(final ChangeEvent e) {
                ZGRViewer.cfgMngr.setSDZoomFactor(PrefWindow.this.sdZoomSlider.getValue());
            }
        };
        this.sdZoomSlider.addChangeListener(cl0);
        final JLabel mFactorLabel = new JLabel("Magnification factor when focusing on a node");
        Utils.buildConstraints(constraints0, 0, 8, 1, 1, 60, 10);
        gridBag0.setConstraints(mFactorLabel, constraints0);
        miscPane.add(mFactorLabel);
        this.mFactorSpinner = new JSpinner(new SpinnerNumberModel(ConfigManager.MAG_FACTOR, 0.1, 10, 0.1));
        Utils.buildConstraints(constraints0, 1, 8, 1, 1, 40, 0);
        gridBag0.setConstraints(this.mFactorSpinner, constraints0);
        miscPane.add(this.mFactorSpinner);

        final JLabel highlightLb = new JLabel("Color of highlighted elements");
        Utils.buildConstraints(constraints0, 0, 9, 1, 1, 60, 10);
        gridBag0.setConstraints(highlightLb, constraints0);
        miscPane.add(highlightLb);
        this.highlightColor = new ColorIndicator(ConfigManager.HIGHLIGHT_COLOR);
        Utils.buildConstraints(constraints0, 1, 9, 1, 1, 40, 0);
        gridBag0.setConstraints(this.highlightColor, constraints0);
        miscPane.add(this.highlightColor);
        this.highlightColor.addMouseListener(this);

        //blank panel to fill remaining part of the tab
        final JPanel p1 = new JPanel();
        Utils.buildConstraints(constraints0, 0, 10, 1, 1, 100, 50);
        gridBag0.setConstraints(p1, constraints0);
        miscPane.add(p1);
        //add tab to panel
        this.tabbedPane.addTab("Misc.", miscPane);

        //directories panel
        final FocusListener fl0 = new FocusListener() {
            @Override
            public void focusGained(final FocusEvent e) {
            }

            @Override
            public void focusLost(final FocusEvent e) {
                final Object src = e.getSource();
                if (src == PrefWindow.this.tmpDirTF) {
                    final File fl = new File(PrefWindow.this.tmpDirTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isDirectory()) {
                            ConfigManager.m_TmpDir = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.tmpDirTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.tmpDirTF.getText());
                    }
                } else if (src == PrefWindow.this.graphDirTF) {
                    final File fl = new File(PrefWindow.this.graphDirTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isDirectory()) {
                            ConfigManager.m_PrjDir = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.graphDirTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.graphDirTF.getText());
                    }
                } else if (src == PrefWindow.this.neatoPathTF) {
                    final File fl = new File(PrefWindow.this.neatoPathTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isFile()) {
                            ConfigManager.m_NeatoPath = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.neatoPathTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.neatoPathTF.getText());
                    }
                } else if (src == PrefWindow.this.circoPathTF) {
                    final File fl = new File(PrefWindow.this.circoPathTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isFile()) {
                            ConfigManager.m_CircoPath = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.circoPathTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.circoPathTF.getText());
                    }
                } else if (src == PrefWindow.this.twopiPathTF) {
                    final File fl = new File(PrefWindow.this.twopiPathTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isFile()) {
                            ConfigManager.m_TwopiPath = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.twopiPathTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.twopiPathTF.getText());
                    }
                } else if (src == PrefWindow.this.dotPathTF) {
                    final File fl = new File(PrefWindow.this.dotPathTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isFile()) {
                            ConfigManager.m_DotPath = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notAFile
                                + PrefWindow.this.dotPathTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.dotPathTF.getText());
                    }
                } else if (src == PrefWindow.this.fontDirTF) {
                    final File fl = new File(PrefWindow.this.fontDirTF.getText().trim());
                    if (fl.exists()) {
                        if (fl.isDirectory()) {
                            ConfigManager.m_GraphVizFontDir = fl;
                        } else {
                            javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.notADirectory
                                + PrefWindow.this.fontDirTF.getText());
                        }
                    } else {
                        javax.swing.JOptionPane.showMessageDialog(PrefWindow.this, Messages.fileDoesNotExist
                            + PrefWindow.this.fontDirTF.getText());
                    }
                }
            }
        };
        final JPanel dirPane = new JPanel();
        final GridBagLayout gridBag = new GridBagLayout();
        final GridBagConstraints constraints = new GridBagConstraints();
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.WEST;
        dirPane.setLayout(gridBag);
        final JLabel l1 = new JLabel("Temporary directory");
        Utils.buildConstraints(constraints, 0, 0, 1, 1, 60, 10);
        gridBag.setConstraints(l1, constraints);
        dirPane.add(l1);
        this.cb1 = new JCheckBox("Delete temp files on exit");
        Utils.buildConstraints(constraints, 1, 0, 1, 1, 30, 0);
        gridBag.setConstraints(this.cb1, constraints);
        if (ConfigManager.DELETE_TEMP_FILES) {
            this.cb1.setSelected(true);
        } else {
            this.cb1.setSelected(false);
        }
        this.cb1.addActionListener(this);
        dirPane.add(this.cb1);
        this.browseTmpDirBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 0, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseTmpDirBt, constraints);
        this.browseTmpDirBt.addActionListener(this);
        dirPane.add(this.browseTmpDirBt);
        this.tmpDirTF = new JTextField(ConfigManager.m_TmpDir.toString());
        Utils.buildConstraints(constraints, 0, 1, 3, 1, 100, 10);
        gridBag.setConstraints(this.tmpDirTF, constraints);
        dirPane.add(this.tmpDirTF);
        this.tmpDirTF.addFocusListener(fl0);
        final JLabel l2 = new JLabel("DOT files directory");
        Utils.buildConstraints(constraints, 0, 2, 2, 1, 90, 10);
        gridBag.setConstraints(l2, constraints);
        dirPane.add(l2);
        this.browseGraphDirBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 2, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseGraphDirBt, constraints);
        this.browseGraphDirBt.addActionListener(this);
        dirPane.add(this.browseGraphDirBt);
        this.graphDirTF = new JTextField(ConfigManager.m_PrjDir.toString());
        Utils.buildConstraints(constraints, 0, 3, 3, 1, 100, 10);
        gridBag.setConstraints(this.graphDirTF, constraints);
        dirPane.add(this.graphDirTF);
        this.graphDirTF.addFocusListener(fl0);
        final JLabel l4 = new JLabel("GraphViz/dot executable");
        Utils.buildConstraints(constraints, 0, 4, 2, 1, 90, 10);
        gridBag.setConstraints(l4, constraints);
        dirPane.add(l4);
        this.browseDotBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 4, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseDotBt, constraints);
        this.browseDotBt.addActionListener(this);
        dirPane.add(this.browseDotBt);
        this.dotPathTF = new JTextField(ConfigManager.m_DotPath.toString());
        Utils.buildConstraints(constraints, 0, 5, 3, 1, 100, 10);
        gridBag.setConstraints(this.dotPathTF, constraints);
        dirPane.add(this.dotPathTF);
        this.dotPathTF.addFocusListener(fl0);
        final JLabel l3 = new JLabel("GraphViz/neato executable");
        Utils.buildConstraints(constraints, 0, 6, 2, 1, 90, 10);
        gridBag.setConstraints(l3, constraints);
        dirPane.add(l3);
        this.browseNeatoBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 6, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseNeatoBt, constraints);
        this.browseNeatoBt.addActionListener(this);
        dirPane.add(this.browseNeatoBt);
        this.neatoPathTF = new JTextField(ConfigManager.m_NeatoPath.toString());
        Utils.buildConstraints(constraints, 0, 7, 3, 1, 100, 10);
        gridBag.setConstraints(this.neatoPathTF, constraints);
        dirPane.add(this.neatoPathTF);
        this.neatoPathTF.addFocusListener(fl0);
        final JLabel l3b = new JLabel("GraphViz/circo executable");
        Utils.buildConstraints(constraints, 0, 8, 2, 1, 90, 10);
        gridBag.setConstraints(l3b, constraints);
        dirPane.add(l3b);
        this.browseCircoBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 8, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseCircoBt, constraints);
        this.browseCircoBt.addActionListener(this);
        dirPane.add(this.browseCircoBt);
        this.circoPathTF = new JTextField(ConfigManager.m_CircoPath.toString());
        Utils.buildConstraints(constraints, 0, 9, 3, 1, 100, 10);
        gridBag.setConstraints(this.circoPathTF, constraints);
        dirPane.add(this.circoPathTF);
        this.circoPathTF.addFocusListener(fl0);
        final JLabel l3c = new JLabel("GraphViz/twopi executable");
        Utils.buildConstraints(constraints, 0, 10, 2, 1, 90, 10);
        gridBag.setConstraints(l3c, constraints);
        dirPane.add(l3c);
        this.browseTwopiBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 10, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseTwopiBt, constraints);
        this.browseTwopiBt.addActionListener(this);
        dirPane.add(this.browseTwopiBt);
        this.twopiPathTF = new JTextField(ConfigManager.m_TwopiPath.toString());
        Utils.buildConstraints(constraints, 0, 11, 3, 1, 100, 10);
        gridBag.setConstraints(this.twopiPathTF, constraints);
        dirPane.add(this.twopiPathTF);
        this.twopiPathTF.addFocusListener(fl0);
        final JLabel l5 = new JLabel("GraphViz font directory (optional)");
        Utils.buildConstraints(constraints, 0, 12, 2, 1, 90, 10);
        gridBag.setConstraints(l5, constraints);
        dirPane.add(l5);
        this.browseFontDirBt = new JButton("Browse...");
        Utils.buildConstraints(constraints, 2, 12, 1, 1, 10, 0);
        gridBag.setConstraints(this.browseFontDirBt, constraints);
        this.browseFontDirBt.addActionListener(this);
        dirPane.add(this.browseFontDirBt);
        this.fontDirTF = new JTextField(ConfigManager.m_GraphVizFontDir.toString());
        Utils.buildConstraints(constraints, 0, 13, 3, 1, 100, 10);
        gridBag.setConstraints(this.fontDirTF, constraints);
        dirPane.add(this.fontDirTF);
        this.fontDirTF.addFocusListener(fl0);
        final JScrollPane dirSP = new JScrollPane(dirPane);
        dirSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        dirSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        this.tabbedPane.addTab("Directories", dirSP);
        //web browser panel
        final JPanel webPane = new JPanel();
        final GridBagLayout gridBag2 = new GridBagLayout();
        final GridBagConstraints constraints2 = new GridBagConstraints();
        constraints2.fill = GridBagConstraints.HORIZONTAL;
        constraints2.anchor = GridBagConstraints.WEST;
        webPane.setLayout(gridBag2);
        final ButtonGroup bg2 = new ButtonGroup();
        this.detectBrowserBt = new JRadioButton("Automatically Detect Default Browser");
        Utils.buildConstraints(constraints2, 0, 0, 3, 1, 100, 1);
        gridBag2.setConstraints(this.detectBrowserBt, constraints2);
        this.detectBrowserBt.addActionListener(this);
        bg2.add(this.detectBrowserBt);
        webPane.add(this.detectBrowserBt);
        this.specifyBrowserBt = new JRadioButton("Specify Browser:");
        Utils.buildConstraints(constraints2, 0, 1, 3, 1, 100, 1);
        gridBag2.setConstraints(this.specifyBrowserBt, constraints2);
        this.specifyBrowserBt.addActionListener(this);
        bg2.add(this.specifyBrowserBt);
        webPane.add(this.specifyBrowserBt);
        final JPanel p7 = new JPanel();
        Utils.buildConstraints(constraints2, 0, 2, 1, 1, 10, 1);
        gridBag2.setConstraints(p7, constraints2);
        webPane.add(p7);
        this.pathLb = new JLabel("Path");
        Utils.buildConstraints(constraints2, 1, 2, 1, 1, 80, 0);
        gridBag2.setConstraints(this.pathLb, constraints2);
        webPane.add(this.pathLb);
        this.brw6 = new JButton("Browse...");
        Utils.buildConstraints(constraints2, 2, 2, 1, 1, 10, 0);
        gridBag2.setConstraints(this.brw6, constraints2);
        this.brw6.addActionListener(this);
        webPane.add(this.brw6);
        this.browserPathTf = new JTextField(ConfigManager.browserPath.toString());
        Utils.buildConstraints(constraints2, 1, 3, 2, 1, 90, 1);
        gridBag2.setConstraints(this.browserPathTf, constraints2);
        webPane.add(this.browserPathTf);
        this.optLb = new JLabel("Command Line Options");
        Utils.buildConstraints(constraints2, 1, 4, 2, 1, 90, 1);
        gridBag2.setConstraints(this.optLb, constraints2);
        webPane.add(this.optLb);
        this.browserOptsTf = new JTextField(ConfigManager.browserOptions);
        Utils.buildConstraints(constraints2, 1, 5, 2, 1, 90, 1);
        gridBag2.setConstraints(this.browserOptsTf, constraints2);
        webPane.add(this.browserOptsTf);
        //fill out empty space
        final JPanel p8 = new JPanel();
        Utils.buildConstraints(constraints2, 0, 6, 3, 1, 100, 92);
        gridBag2.setConstraints(p8, constraints2);
        webPane.add(p8);
        this.webHelpBt = new JButton("Help");
        Utils.buildConstraints(constraints2, 2, 7, 1, 1, 10, 1);
        gridBag2.setConstraints(this.webHelpBt, constraints2);
        this.webHelpBt.addActionListener(this);
        webPane.add(this.webHelpBt);
        if (ConfigManager.autoDetectBrowser) {
            this.detectBrowserBt.doClick();
        } //select and fire event
        else {
            this.specifyBrowserBt.doClick();
        } //so that fields get enabled/disabled as is approriate
        this.tabbedPane.addTab("Web Browser", webPane);

        //proxy panel
        final JPanel proxyPane = new JPanel();
        final GridBagLayout gridBag5 = new GridBagLayout();
        final GridBagConstraints constraints5 = new GridBagConstraints();
        constraints5.fill = GridBagConstraints.HORIZONTAL;
        constraints5.anchor = GridBagConstraints.WEST;
        proxyPane.setLayout(gridBag5);
        this.useProxyCb = new JCheckBox("Use Proxy Server");
        Utils.buildConstraints(constraints5, 0, 0, 2, 1, 100, 1);
        gridBag5.setConstraints(this.useProxyCb, constraints5);
        this.useProxyCb.setSelected(ConfigManager.useProxy);
        this.useProxyCb.addActionListener(this);
        proxyPane.add(this.useProxyCb);
        this.proxyHostLb = new JLabel("Hostname:");
        this.proxyHostLb.setEnabled(ConfigManager.useProxy);
        Utils.buildConstraints(constraints5, 0, 1, 1, 1, 80, 1);
        gridBag5.setConstraints(this.proxyHostLb, constraints5);
        proxyPane.add(this.proxyHostLb);
        this.proxyPortLb = new JLabel("Port:");
        this.proxyPortLb.setEnabled(ConfigManager.useProxy);
        Utils.buildConstraints(constraints5, 1, 1, 1, 1, 20, 1);
        gridBag5.setConstraints(this.proxyPortLb, constraints5);
        proxyPane.add(this.proxyPortLb);
        this.proxyHostTf = new JTextField(ConfigManager.proxyHost);
        this.proxyHostTf.setEnabled(ConfigManager.useProxy);
        Utils.buildConstraints(constraints5, 0, 2, 1, 1, 80, 1);
        gridBag5.setConstraints(this.proxyHostTf, constraints5);
        proxyPane.add(this.proxyHostTf);
        this.proxyPortTf = new JTextField(ConfigManager.proxyPort);
        this.proxyPortTf.setEnabled(ConfigManager.useProxy);
        Utils.buildConstraints(constraints5, 1, 2, 1, 1, 20, 1);
        gridBag5.setConstraints(this.proxyPortTf, constraints5);
        proxyPane.add(this.proxyPortTf);
        constraints5.fill = GridBagConstraints.BOTH;
        constraints5.anchor = GridBagConstraints.CENTER;
        //fill out empty space
        final JPanel p1000 = new JPanel();
        Utils.buildConstraints(constraints5, 0, 5, 2, 1, 100, 90);
        gridBag5.setConstraints(p1000, constraints5);
        proxyPane.add(p1000);
        constraints5.fill = GridBagConstraints.NONE;
        constraints5.anchor = GridBagConstraints.EAST;
        this.proxyHelpBt = new JButton("Help");
        Utils.buildConstraints(constraints5, 1, 6, 1, 1, 20, 1);
        gridBag5.setConstraints(this.proxyHelpBt, constraints5);
        this.proxyHelpBt.addActionListener(this);
        proxyPane.add(this.proxyHelpBt);
        this.tabbedPane.addTab("Proxy", proxyPane);

        //plugin panel
        this.tabbedPane.addTab("Plugins", initPluginPane());

        //main panel (tabbed panes + OK/Save buttons)
        final Container cpane = this.getContentPane();
        final GridBagLayout gridBag3 = new GridBagLayout();
        final GridBagConstraints constraints3 = new GridBagConstraints();
        constraints3.fill = GridBagConstraints.BOTH;
        constraints3.anchor = GridBagConstraints.WEST;
        cpane.setLayout(gridBag3);
        Utils.buildConstraints(constraints3, 0, 0, 3, 1, 100, 90);
        gridBag3.setConstraints(this.tabbedPane, constraints3);
        cpane.add(this.tabbedPane);
        final JPanel tmp = new JPanel();
        Utils.buildConstraints(constraints3, 0, 1, 1, 1, 70, 10);
        gridBag3.setConstraints(tmp, constraints3);
        cpane.add(tmp);
        constraints3.fill = GridBagConstraints.HORIZONTAL;
        constraints3.anchor = GridBagConstraints.CENTER;
        this.okPrefs = new JButton("Apply & Close");
        //okPrefs.setPreferredSize(new Dimension(60,25));
        Utils.buildConstraints(constraints3, 1, 1, 1, 1, 15, 10);
        gridBag3.setConstraints(this.okPrefs, constraints3);
        this.okPrefs.addActionListener(this);
        cpane.add(this.okPrefs);
        constraints3.fill = GridBagConstraints.HORIZONTAL;
        constraints3.anchor = GridBagConstraints.CENTER;
        this.savePrefs = new JButton("Save");
        //savePrefs.setPreferredSize(new Dimension(60,35));
        Utils.buildConstraints(constraints3, 2, 1, 1, 1, 15, 10);
        gridBag3.setConstraints(this.savePrefs, constraints3);
        this.savePrefs.addActionListener(this);
        cpane.add(this.savePrefs);

        this.tabbedPane.setSelectedIndex(0);
        //window
        final WindowListener w0 = new WindowAdapter() {
            @Override
            public void windowClosing(final WindowEvent e) {
            }
        };
        this.addWindowListener(w0);
        this.setTitle("Preferences");
        this.pack();
        this.setSize(400, 360);
    }

    private JComponent initPluginPane() {
        final JPanel pluginPane = new JPanel();
        final int nbPlugins = ZGRViewer.cfgMngr.plugins.length;

        final GridBagLayout gridBag = new GridBagLayout();
        final GridBagConstraints constraints = new GridBagConstraints();
        constraints.fill = GridBagConstraints.NONE;
        pluginPane.setLayout(gridBag);
        JButton b65;
        JLabel l65;
        ActionListener a65;
        MouseListener m65;
        int row = 0;
        for (int i = 0; i < nbPlugins; i++) {
            row = i * 2;
            constraints.anchor = GridBagConstraints.WEST;
            final String url =
                (ZGRViewer.cfgMngr.plugins[i].getURL() != null) ? ZGRViewer.cfgMngr.plugins[i].getURL().toString() : "";
            l65 = new JLabel("<html><a href=\"" + url + "\">" + ZGRViewer.cfgMngr.plugins[i].getName() + "</a></html>");
            Utils.buildConstraints(constraints, 0, row, 1, 1, 70, 5);
            gridBag.setConstraints(l65, constraints);
            pluginPane.add(l65);
            m65 = new MouseListener() {
                @Override
                public void mouseClicked(final MouseEvent e) {
                    if (url != null && url.length() > 0) {
                        PrefWindow.this.application.displayURLinBrowser(url);
                    }
                }

                @Override
                public void mouseEntered(final MouseEvent e) {
                }

                @Override
                public void mouseExited(final MouseEvent e) {
                }

                @Override
                public void mousePressed(final MouseEvent e) {
                }

                @Override
                public void mouseReleased(final MouseEvent e) {
                }
            };
            l65.addMouseListener(m65);
            constraints.anchor = GridBagConstraints.EAST;
            b65 = new JButton("Settings...");
            Utils.buildConstraints(constraints, 1, row, 1, 1, 30, 0);
            gridBag.setConstraints(b65, constraints);
            pluginPane.add(b65);
            final Plugin pg65 = ZGRViewer.cfgMngr.plugins[i];
            a65 = new ActionListener() {
                @Override
                public void actionPerformed(final ActionEvent e) {
                    pg65.showSettings();
                }
            };
            b65.addActionListener(a65);
            constraints.anchor = GridBagConstraints.WEST;
            l65 = new JLabel("Author: " + ZGRViewer.cfgMngr.plugins[i].getAuthor());
            Utils.buildConstraints(constraints, 0, row + 1, 1, 1, 0, 5);
            gridBag.setConstraints(l65, constraints);
            pluginPane.add(l65);
            constraints.anchor = GridBagConstraints.EAST;
            l65 = new JLabel("Version: " + ZGRViewer.cfgMngr.plugins[i].getVersion());
            Utils.buildConstraints(constraints, 0, row + 1, 1, 1, 0, 0);
            gridBag.setConstraints(l65, constraints);
            pluginPane.add(l65);
        }
        final JPanel p47 = new JPanel();
        constraints.fill = GridBagConstraints.BOTH;
        constraints.anchor = GridBagConstraints.WEST;
        Utils.buildConstraints(constraints, 0, row + 2, 2, 1, 100, 90);
        gridBag.setConstraints(p47, constraints);
        pluginPane.add(p47);
        return new JScrollPane(
            pluginPane,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    }

    @Override
    public void actionPerformed(final ActionEvent e) {
        JFileChooser fc;
        int returnVal;
        final Object o = e.getSource();
        if (o == this.browseTmpDirBt) {
            //tmp directory browse button
            fc = new JFileChooser(ConfigManager.m_TmpDir);
            fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_TmpDir = fc.getSelectedFile();
                this.tmpDirTF.setText(ConfigManager.m_TmpDir.toString());
            }
        } else if (o == this.browseGraphDirBt) {
            fc = new JFileChooser(ConfigManager.m_PrjDir);
            fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_PrjDir = fc.getSelectedFile();
                this.graphDirTF.setText(ConfigManager.m_PrjDir.toString());
            }
        } else if (o == this.browseDotBt) {
            fc = new JFileChooser(ConfigManager.m_DotPath);
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_DotPath = fc.getSelectedFile();
                this.dotPathTF.setText(ConfigManager.m_DotPath.toString());
            }
        } else if (o == this.browseNeatoBt) {
            fc = new JFileChooser(ConfigManager.m_NeatoPath);
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_NeatoPath = fc.getSelectedFile();
                this.neatoPathTF.setText(ConfigManager.m_NeatoPath.toString());
            }
        } else if (o == this.browseCircoBt) {
            fc = new JFileChooser(ConfigManager.m_CircoPath);
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_CircoPath = fc.getSelectedFile();
                this.circoPathTF.setText(ConfigManager.m_CircoPath.toString());
            }
        } else if (o == this.browseTwopiBt) {
            fc = new JFileChooser(ConfigManager.m_TwopiPath);
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_TwopiPath = fc.getSelectedFile();
                this.twopiPathTF.setText(ConfigManager.m_TwopiPath.toString());
            }
        } else if (o == this.browseFontDirBt) {
            fc = new JFileChooser(ConfigManager.m_GraphVizFontDir);
            fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.m_GraphVizFontDir = fc.getSelectedFile();
                this.fontDirTF.setText(ConfigManager.m_GraphVizFontDir.toString());
            }
        } else if (o == this.cb1) {
            if (this.cb1.isSelected()) {
                ConfigManager.DELETE_TEMP_FILES = true;
            } else {
                ConfigManager.DELETE_TEMP_FILES = false;
            }
        } else if (o == this.detectBrowserBt) {
            if (this.detectBrowserBt.isSelected()) {
                //automatically detect browser
                ConfigManager.autoDetectBrowser = true;
                this.browserPathTf.setEnabled(false);
                this.brw6.setEnabled(false);
                this.browserOptsTf.setEnabled(false);
                this.pathLb.setEnabled(false);
                this.optLb.setEnabled(false);
            }
        } else if (o == this.specifyBrowserBt) {
            if (this.specifyBrowserBt.isSelected()) {
                //specify browser
                ConfigManager.autoDetectBrowser = false;
                this.browserPathTf.setEnabled(true);
                this.brw6.setEnabled(true);
                this.browserOptsTf.setEnabled(true);
                this.pathLb.setEnabled(true);
                this.optLb.setEnabled(true);
            }
        } else if (o == this.brw6) {
            fc = new JFileChooser(ConfigManager.browserPath);
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            returnVal = fc.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                ConfigManager.browserPath = fc.getSelectedFile();
                this.browserPathTf.setText(ConfigManager.browserPath.toString());
            }
        } else if (o == this.webHelpBt) {
            final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            final TextViewer help =
                new TextViewer(
                    new StringBuffer(Messages.webBrowserHelpText),
                    "Web Browser Configuration",
                    0,
                    (screenSize.width - 400) / 2,
                    (screenSize.height - 300) / 2,
                    400,
                    300,
                    false);
        } else if (o == this.useProxyCb) {
            this.proxyHostLb.setEnabled(this.useProxyCb.isSelected());
            this.proxyPortLb.setEnabled(this.useProxyCb.isSelected());
            this.proxyHostTf.setEnabled(this.useProxyCb.isSelected());
            this.proxyPortTf.setEnabled(this.useProxyCb.isSelected());
        } else if (o == this.proxyHelpBt) {
            final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            final TextViewer help =
                new TextViewer(
                    new StringBuffer(Messages.proxyHelpText),
                    "Proxy Configuration",
                    0,
                    (screenSize.width - 400) / 2,
                    (screenSize.height - 300) / 2,
                    400,
                    300,
                    false);
        } else if (o == this.okPrefs) {
            updateVars();
            this.dispose();
        } else if (o == this.savePrefs) {
            updateVars();
            this.application.saveConfiguration();
        } else if (o == this.antialiascb) {
            if (this.antialiascb.isSelected()) {
                javax.swing.JOptionPane.showMessageDialog(this, Messages.antialiasingWarning);
            }
            this.grMngr.setAntialiasing(this.antialiascb.isSelected());
        } else if (o == this.dynaspotCb) {
            this.grMngr.activateDynaSpot(this.dynaspotCb.isSelected(), true);
        }
    }

    @Override
    public void mouseClicked(final MouseEvent e) {
        final Object o = e.getSource();
        if (o == this.highlightColor) {
            final Color newCol = JColorChooser.showDialog(this, "Highlight Color", this.highlightColor.getColor());
            if (newCol != null) {
                ConfigManager.HIGHLIGHT_COLOR = newCol;
                this.highlightColor.setColor(ConfigManager.HIGHLIGHT_COLOR);
                Glyph.setDefaultCursorInsideHighlightColor(ConfigManager.HIGHLIGHT_COLOR);
            }
        }
    }

    @Override
    public void mousePressed(final MouseEvent e) {
    }

    @Override
    public void mouseReleased(final MouseEvent e) {
    }

    @Override
    public void mouseEntered(final MouseEvent e) {
    }

    @Override
    public void mouseExited(final MouseEvent e) {
    }

    void updateVars() {
        ConfigManager.SAVE_WINDOW_LAYOUT = this.saveWindowLayoutCb.isSelected();
        ConfigManager.FORCE_SILENT = this.silentCb.isSelected();
        ConfigManager.MAG_FACTOR = ((Double) this.mFactorSpinner.getValue()).floatValue();
        ConfigManager.CMD_LINE_OPTS = this.cmdLOptsTf.getText();
        ConfigManager.browserPath = new File(this.browserPathTf.getText());
        ConfigManager.browserOptions = this.browserOptsTf.getText();
        ConfigManager.updateProxy(this.useProxyCb.isSelected(), this.proxyHostTf.getText(), this.proxyPortTf.getText());
    }

}

class ColorIndicator extends JPanel {

    Color color;

    ColorIndicator(final Color c) {
        super();
        this.color = c;
        setBorder(BorderFactory.createLineBorder(Color.BLACK));
        this.setBackground(this.color);
    }

    void setColor(final Color c) {
        this.color = c;
        this.setBackground(this.color);
        repaint();
    }

    Color getColor() {
        return this.color;
    }

}
