/*   FILE: ZgrAppletEvtHdlr.java
 *   DATE OF CREATION:   Fri May 09 09:54:03 2003
 *   Copyright (c) 2003 World Wide Web Consortium. All Rights Reserved
 *   Copyright (c) INRIA, 2004-2011. All Rights Reserved
 *   Licensed under the GNU LGPL. For full terms see the file COPYING.
 *
 *$Id$
 */

package net.claribole.zgrviewer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import fr.inria.zvtm.animation.*;
import fr.inria.zvtm.animation.interpolation.*;
import fr.inria.zvtm.engine.*;
import fr.inria.zvtm.event.*;
import fr.inria.zvtm.glyphs.*;

public class ZgrAppletEvtHdlr extends BaseEventHandler implements ViewListener {

    protected ZGRApplet application;

    protected ZgrAppletEvtHdlr(final ZGRApplet app, final GraphicsManager gm) {
        this.application = app;
        this.grMngr = gm;
    }

    @Override
    public void press1(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
        if (this.toolPaletteIsActive) {
            return;
        }
        this.lastJPX = jpx;
        this.lastJPY = jpy;
        Glyph g = v.lastGlyphEntered();
        if (this.inZoomWindow) {
            if (this.grMngr.dmPortal.coordInsideBar(jpx, jpy)) {
                this.draggingZoomWindow = true;
            } else {
                this.draggingZoomWindowContent = true;
            }
        } else if (this.inMagWindow) {
            v.getVCursor().stickGlyph(this.grMngr.magWindow);
            this.draggingMagWindow = true;
        } else if (this.grMngr.tp.isBringAndGoMode() && (g = v.lastGlyphEntered()) != null) {
            this.grMngr.startBringAndGo(g);
        } else if (this.grMngr.tp.isLinkSlidingMode()) {
            // link sliding is not possible in Applet mode (java.awt.Robot security exception)
            //Point location = e.getComponent().getLocationOnScreen();
            //relative = e.getPoint();
            //LS_SX = v.getVCursor().vx;
            //LS_SY = v.getVCursor().vy;
            //grMngr.attemptLinkSliding(LS_SX, LS_SY, location.x, location.y);
        } else if (this.grMngr.tp.isEditMode()) {
            pressInEditMode(g, v.getVCursor(), this.grMngr.mainCamera, mod);
        } else {
            this.grMngr.rememberLocation(v.cams[0].getLocation());
            if (mod == NO_MODIFIER || mod == SHIFT_MOD || mod == META_MOD || mod == META_SHIFT_MOD) {
                this.manualLeftButtonMove = true;
                this.lastJPX = jpx;
                this.lastJPY = jpy;
                //grMngr.vsm.setActiveCamera(v.cams[0]);
                v.showFirstOrderPanWidget(jpx, jpy);
                //v.setDrawDrag(true);
                v.getVCursor().setSensitivity(false); //because we would not be consistent  (when dragging the mouse, we computeMouseOverList, but if there is an anim triggered by {X,Y,A}speed, and if the mouse is not moving, this list is not computed - so here we choose to disable this computation when dragging the mouse with button 3 pressed)
                this.activeCam = this.grMngr.vsm.getActiveCamera();
            } else if (mod == ALT_MOD) {
                this.zoomingInRegion = true;
                this.x1 = v.getVCursor().getVSXCoordinate();
                this.y1 = v.getVCursor().getVSYCoordinate();
                v.setDrawRect(true);
            }
        }
    }

    @Override
    public void release1(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
        if (this.grMngr.isBringingAndGoing) {
            this.grMngr.endBringAndGo(v.lastGlyphEntered());
        } else if (this.grMngr.isLinkSliding) {
            // link sliding is not possible in Applet mode (java.awt.Robot security exception)
            //grMngr.endLinkSliding();
        }
        if (this.toolPaletteIsActive) {
            return;
        }
        this.draggingZoomWindow = false;
        this.draggingZoomWindowContent = false;
        if (this.editingSpline || this.movingEdgeLabelOrBox) {
            v.getVCursor().unstickLastGlyph();
            this.editingSpline = this.movingEdgeLabelOrBox = false;
        } else if (this.movingNode) {
            v.getVCursor().unstickLastGlyph();
            this.grMngr.geom.unstickAll();
            this.movingNode = false;
        }
        if (this.draggingMagWindow) {
            this.draggingMagWindow = false;
            v.getVCursor().unstickLastGlyph();
        }
        if (this.zoomingInRegion) {
            v.setDrawRect(false);
            this.x2 = v.getVCursor().getVSXCoordinate();
            this.y2 = v.getVCursor().getVSYCoordinate();
            if ((Math.abs(this.x2 - this.x1) >= 4) && (Math.abs(this.y2 - this.y1) >= 4)) {
                this.grMngr.mainView.centerOnRegion(
                    this.grMngr.vsm.getActiveCamera(),
                    ConfigManager.ANIM_MOVE_LENGTH,
                    this.x1,
                    this.y1,
                    this.x2,
                    this.y2);
            }
            this.zoomingInRegion = false;
        } else if (this.manualLeftButtonMove) {
            this.grMngr.mainCamera.setXspeed(0);
            this.grMngr.mainCamera.setYspeed(0);
            this.grMngr.mainCamera.setZspeed(0);
            v.hideFirstOrderPanWidget();
            //v.setDrawDrag(false);
            v.getVCursor().setSensitivity(true);
            if (this.autoZooming) {
                unzoom(v);
            }
            this.manualLeftButtonMove = false;
        }
    }

    @Override
    public void click1(
        final ViewPanel v,
        final int mod,
        final int jpx,
        final int jpy,
        final int clickNumber,
        final MouseEvent e)
    {
        if (this.toolPaletteIsActive) {
            if (v.lastGlyphEntered() != null) {
                this.grMngr.tp.selectMode((VImage) v.lastGlyphEntered());
            }
        } else if (mod == META_MOD) {
            if (this.grMngr.tp.isHighlightMode() && v.getGlyphsUnderCursorList().length > 0) {
                final Glyph g = v.lastGlyphEntered();
                this.grMngr.highlightElement(g, v.cams[0], v.getVCursor(), true, 0, true, -1);
            }
        } else {
            if (this.grMngr.tp.isBringAndGoMode() || this.grMngr.tp.isLinkSlidingMode()) {
                return;
            }
            if (this.grMngr.tp.isFadingLensNavMode() || this.grMngr.tp.isProbingLensNavMode()) {
                this.lastJPX = jpx;
                this.lastJPY = jpy;
                this.lastVX = v.getVCursor().getVSXCoordinate();
                this.lastVY = v.getVCursor().getVSYCoordinate();
                if (this.grMngr.lensType != GraphicsManager.NO_LENS) {
                    this.grMngr.zoomInPhase2(this.lastVX, this.lastVY);
                } else {
                    if (this.cursorNearBorder) {
                        // do not activate the lens when cursor is near the border
                        return;
                    }
                    this.grMngr.zoomInPhase1(jpx, jpy);
                }
            } else if (this.grMngr.tp.isDragMagNavMode()) {
                this.grMngr.triggerDM(jpx, jpy, this);
            } else if (this.grMngr.tp.isEditMode()) {
                return;
            } else {
                if (clickNumber == 2) {
                    click2(v, mod, jpx, jpy, clickNumber, e);
                } else {
                    final Glyph g = v.lastGlyphEntered();
                    if (mod == SHIFT_MOD) {
                        this.grMngr.highlightElement(g, v.cams[0], v.getVCursor(), true, 0, true, -1);
                    } else {
                        if (g != null && g != this.grMngr.boundingBox) {
                            this.grMngr.mainView.centerOnGlyph(
                                g,
                                v.cams[0],
                                ConfigManager.ANIM_MOVE_LENGTH,
                                true,
                                ConfigManager.MAG_FACTOR);
                        }
                    }
                }
            }
        }
    }

    @Override
    public void press2(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
        this.grMngr.paMngr.requestToolPaletteRelocation();
    }

    @Override
    public void release2(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
    }

    @Override
    public void click2(
        final ViewPanel v,
        final int mod,
        final int jpx,
        final int jpy,
        final int clickNumber,
        final MouseEvent e)
    {
        if (this.toolPaletteIsActive) {
            return;
        }
        final Glyph g = v.lastGlyphEntered();
        if (g != null && g != this.grMngr.boundingBox) {
            if (g.getOwner() != null) {
                getAndDisplayURL((LElem) g.getOwner(), g);
            }
        } else {
            attemptDisplayEdgeURL(v.getVCursor(), v.cams[0]);
        }
    }

    @Override
    public void press3(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
        if (this.toolPaletteIsActive) {
            return;
        } else {
            if (this.grMngr.tp.isFadingLensNavMode() || this.grMngr.tp.isProbingLensNavMode()) {
                this.lastJPX = jpx;
                this.lastJPY = jpy;
            }
        }
    }

    @Override
    public void release3(final ViewPanel v, final int mod, final int jpx, final int jpy, final MouseEvent e) {
    }

    @Override
    public void click3(
        final ViewPanel v,
        final int mod,
        final int jpx,
        final int jpy,
        final int clickNumber,
        final MouseEvent e)
    {
        if (this.toolPaletteIsActive) {
            return;
        } else {
            if (this.grMngr.tp.isFadingLensNavMode() || this.grMngr.tp.isProbingLensNavMode()) {
                this.lastJPX = jpx;
                this.lastJPY = jpy;
                this.lastVX = v.getVCursor().getVSXCoordinate();
                this.lastVY = v.getVCursor().getVSYCoordinate();
                if (this.grMngr.lensType != GraphicsManager.NO_LENS) {
                    this.grMngr.zoomOutPhase2();
                } else {
                    if (this.cursorNearBorder) {
                        // do not activate the lens when cursor is near the border
                        return;
                    }
                    this.grMngr.zoomOutPhase1(jpx, jpy, this.lastVX, this.lastVY);
                }
            }
        }
    }

    @Override
    public void mouseMoved(final ViewPanel v, final int jpx, final int jpy, final MouseEvent e) {
        this.lx = jpx;
        this.ly = jpy;
        if ((jpx - GraphicsManager.LENS_R1) < 0) {
            this.lx = GraphicsManager.LENS_R1;
            this.cursorNearBorder = true;
        } else if ((jpx + GraphicsManager.LENS_R1) > this.grMngr.panelWidth) {
            this.lx = this.grMngr.panelWidth - GraphicsManager.LENS_R1;
            this.cursorNearBorder = true;
        } else {
            this.cursorNearBorder = false;
        }
        if ((jpy - GraphicsManager.LENS_R1) < 0) {
            this.ly = GraphicsManager.LENS_R1;
            this.cursorNearBorder = true;
        } else if ((jpy + GraphicsManager.LENS_R1) > this.grMngr.panelHeight) {
            this.ly = this.grMngr.panelHeight - GraphicsManager.LENS_R1;
            this.cursorNearBorder = true;
        }
        if (this.grMngr.lensType != 0 && this.grMngr.lens != null) {
            this.grMngr.moveLens(this.lx, this.ly, e.getWhen());
        } else if (this.grMngr.tp.isEnabled()) {
            if (this.grMngr.tp.insidePaletteTriggerZone(jpx, jpy)) {
                if (!this.grMngr.tp.isShowing()) {
                    this.grMngr.tp.show();
                    this.toolPaletteIsActive = true;
                }
            } else {
                if (this.grMngr.tp.isShowing()) {
                    this.grMngr.tp.hide();
                    this.toolPaletteIsActive = false;
                }
            }
        }
    }

    @Override
    public void mouseDragged(
        final ViewPanel v,
        final int mod,
        final int buttonNumber,
        final int jpx,
        final int jpy,
        final MouseEvent e)
    {
        if (this.toolPaletteIsActive || this.grMngr.isBringingAndGoing) {
            return;
        }
        // link sliding is not possible in Applet mode (java.awt.Robot security exception)
        //if (grMngr.isLinkSliding){
        //	// ignore events triggered by AWT robot
        //	grMngr.linkSlider(v.getVCursor().vx, v.getVCursor().vy, false);
        //}
        if (this.editingSpline) {
            this.grMngr.geom.updateEdgeSpline();
        } else if (this.movingEdgeLabelOrBox || this.movingNode) {
            // do nothing but prevent exec of else
            return;
        } else if (mod != ALT_MOD && buttonNumber == 1) {
            if (this.draggingZoomWindow) {
                this.grMngr.dmPortal.move(jpx - this.lastJPX, jpy - this.lastJPY);
                this.lastJPX = jpx;
                this.lastJPY = jpy;
                this.grMngr.vsm.repaint();
            } else if (this.draggingZoomWindowContent) {
                this.tfactor =
                    (this.grMngr.dmCamera.focal + (this.grMngr.dmCamera.altitude)) / this.grMngr.dmCamera.focal;
                synchronized (this.grMngr.dmCamera) {
                    this.grMngr.dmCamera.move(
                        Math.round(this.tfactor * (this.lastJPX - jpx)),
                        Math.round(this.tfactor * (jpy - this.lastJPY)));
                    this.lastJPX = jpx;
                    this.lastJPY = jpy;
                    this.grMngr.updateMagWindow();
                }
            } else if (this.draggingMagWindow) {
                this.grMngr.updateZoomWindow();
            } else if (this.manualLeftButtonMove) {
                if (!v.isShowingFirstOrderPanWidget()) {
                    v.showFirstOrderPanWidget(this.lastJPX, this.lastJPY);
                }
                if (mod == SHIFT_MOD || mod == META_SHIFT_MOD) {
                    this.grMngr.mainCamera.setXspeed(0);
                    this.grMngr.mainCamera.setYspeed(0);
                    this.grMngr.mainCamera.setZspeed((this.lastJPY - jpy) * ZOOM_SPEED_COEF);
                } else {
                    this.tfactor = (this.activeCam.focal + Math.abs(this.activeCam.altitude)) / this.activeCam.focal;
                    this.jpxD = jpx - this.lastJPX;
                    this.jpyD = this.lastJPY - jpy;
                    this.grMngr.mainCamera.setXspeed((this.activeCam.altitude > 0) ? this.jpxD
                        * (this.tfactor / PAN_SPEED_FACTOR) : this.jpxD / (this.tfactor * PAN_SPEED_FACTOR));
                    this.grMngr.mainCamera.setYspeed((this.activeCam.altitude > 0) ? this.jpyD
                        * (this.tfactor / PAN_SPEED_FACTOR) : this.jpyD / (this.tfactor * PAN_SPEED_FACTOR));
                    this.grMngr.mainCamera.setZspeed(0);
                    if (ZGRApplet.cfgMngr.isSDZoomEnabled()) {
                        this.dragValue = Math.sqrt(Math.pow(this.jpxD, 2) + Math.pow(this.jpyD, 2));
                        if (!this.autoZooming && this.dragValue > ZGRApplet.cfgMngr.SD_ZOOM_THRESHOLD) {
                            this.autoZooming = true;
                            final Animation a =
                                this.grMngr.vsm
                                    .getAnimationManager()
                                    .getAnimationFactory()
                                    .createCameraAltAnim(
                                        300,
                                        v.cams[0],
                                        new Float(ZGRApplet.cfgMngr.autoZoomFactor
                                            * (v.cams[0].getAltitude() + v.cams[0].getFocal())),
                                        true,
                                        IdentityInterpolator.getInstance(),
                                        null);
                            this.grMngr.vsm.getAnimationManager().startAnimation(a, false);
                        }
                    }
                }
            }
        }
        if (this.grMngr.lensType != GraphicsManager.NO_LENS && this.grMngr.lens != null) {
            this.grMngr.moveLens(jpx, jpy, e.getWhen());
        }
    }

    @Override
    public void mouseWheelMoved(
        final ViewPanel v,
        final short wheelDirection,
        final int jpx,
        final int jpy,
        final MouseWheelEvent e)
    {
        if (this.grMngr.lensType != GraphicsManager.NO_LENS && this.grMngr.lens != null) {
            if (wheelDirection == ViewListener.WHEEL_UP) {
                this.grMngr.magnifyFocus(GraphicsManager.WHEEL_MM_STEP, this.grMngr.lensType, this.grMngr.mainCamera);
            } else {
                this.grMngr.magnifyFocus(-GraphicsManager.WHEEL_MM_STEP, this.grMngr.lensType, this.grMngr.mainCamera);
            }
        } else if (this.grMngr.tp.isHighlightMode() && v.getGlyphsUnderCursorList().length > 0) {
            final Glyph g = v.lastGlyphEntered();
            int dir = -1;
            if (e.isShiftDown()) {
                dir = LEdge.INCOMING;
            }
            if (e.isControlDown()) {
                dir = LEdge.OUTGOING;
            }
            if (e.isShiftDown() && e.isControlDown()) {
                dir = LEdge.UNDIRECTED;
            }
            this.grMngr.highlightElement(
                g,
                null,
                null,
                true,
                (wheelDirection == ViewListener.WHEEL_UP) ? -1 : 1,
                false,
                dir); // g is guaranteed to be != null, don't care about camera and cursor
        } else if (this.inZoomWindow) {
            this.tfactor =
                (this.grMngr.dmCamera.focal + Math.abs(this.grMngr.dmCamera.altitude)) / this.grMngr.dmCamera.focal;
            if (wheelDirection == WHEEL_UP) {
                // zooming out
                this.grMngr.dmCamera.altitudeOffset(-this.tfactor * WHEEL_ZOOMOUT_FACTOR);
            } else {
                // wheelDirection == WHEEL_DOWN, zooming in
                this.grMngr.dmCamera.altitudeOffset(this.tfactor * WHEEL_ZOOMIN_FACTOR);
            }
            this.grMngr.updateMagWindow();
            this.grMngr.vsm.repaint();
        } else {
            this.tfactor =
                (this.grMngr.mainCamera.focal + Math.abs(this.grMngr.mainCamera.altitude))
                    / this.grMngr.mainCamera.focal;
            this.mvx = v.getVCursor().getVSXCoordinate();
            this.mvy = v.getVCursor().getVSYCoordinate();
            if (wheelDirection == WHEEL_UP) {
                // zooming out
                this.grMngr.mainCamera.vx -=
                    Math.round((this.mvx - this.grMngr.mainCamera.vx)
                        * WHEEL_ZOOMOUT_FACTOR
                        / this.grMngr.mainCamera.focal);
                this.grMngr.mainCamera.vy -=
                    Math.round((this.mvy - this.grMngr.mainCamera.vy)
                        * WHEEL_ZOOMOUT_FACTOR
                        / this.grMngr.mainCamera.focal);
                this.grMngr.mainCamera.altitudeOffset(this.tfactor * WHEEL_ZOOMOUT_FACTOR);
                this.grMngr.cameraMoved(null, null, 0);
            } else {
                // wheelDirection == WHEEL_DOWN, zooming in
                if (this.grMngr.mainCamera.getAltitude() > -90) {
                    this.grMngr.mainCamera.vx +=
                        Math.round((this.mvx - this.grMngr.mainCamera.vx)
                            * WHEEL_ZOOMIN_FACTOR
                            / this.grMngr.mainCamera.focal);
                    this.grMngr.mainCamera.vy +=
                        Math.round((this.mvy - this.grMngr.mainCamera.vy)
                            * WHEEL_ZOOMIN_FACTOR
                            / this.grMngr.mainCamera.focal);
                }
                this.grMngr.mainCamera.altitudeOffset(-this.tfactor * WHEEL_ZOOMIN_FACTOR);
                this.grMngr.cameraMoved(null, null, 0);
            }
        }
    }

    @Override
    public void enterGlyph(final Glyph g) {
        this.grMngr.mainView.setStatusBarText(Messages.SPACE_STRING);
        if (g == this.grMngr.magWindow) {
            this.inMagWindow = true;
            return;
        }
        if (g == this.grMngr.boundingBox) {
            return;
        } // do not highlight graph's bounding box
        if (g.getType() != null && g.getType().equals(GeometryEditor.SPLINE_GEOM_EDITOR)) {
            this.grMngr.mainView.setCursorIcon(Cursor.MOVE_CURSOR);
        } else if (this.grMngr.tp.isHighlightMode()) {
            this.grMngr.highlightElement(g, null, null, true, 0, false, -1); // g is guaranteed to be != null, don't care about camera and cursor
        } else {
            // node highlighting is taken care of above (in a slightly different manner)
            g.highlight(true, null);
        }
    }

    @Override
    public void exitGlyph(final Glyph g) {
        if (g == this.grMngr.magWindow) {
            this.inMagWindow = false;
            return;
        }
        // do not highlight graph's bounding box
        if (g == this.grMngr.boundingBox) {
            return;
        }
        if (g.getType() != null && g.getType().equals(GeometryEditor.SPLINE_GEOM_EDITOR)) {
            this.grMngr.mainView.setCursorIcon(Cursor.CUSTOM_CURSOR);
        } else if (this.application.grMngr.tp.isHighlightMode()) {
            this.grMngr.unhighlightAll();
        } else {
            // node highlighting is taken care of above (in a slightly different manner)
            g.highlight(false, null);
        }
    }

    @Override
    public void Ktype(final ViewPanel v, final char c, final int code, final int mod, final KeyEvent e) {
        this.application.keyTyped(e);
    }

    @Override
    public void Kpress(final ViewPanel v, final char c, final int code, final int mod, final KeyEvent e) {
        this.application.keyPressed(e);
    }

    @Override
    public void Krelease(final ViewPanel v, final char c, final int code, final int mod, final KeyEvent e) {
        this.application.keyReleased(e);
    }

    @Override
    public void viewActivated(final View v) {
    }

    @Override
    public void viewDeactivated(final View v) {
    }

    @Override
    public void viewIconified(final View v) {
    }

    @Override
    public void viewDeiconified(final View v) {
    }

    @Override
    public void viewClosing(final View v) {
    }

    protected void attemptDisplayEdgeURL(final VCursor mouse, final Camera cam) {
        Glyph g;
        final Vector otherGlyphs = mouse.getPicker().getIntersectingGlyphs(cam);
        if (otherGlyphs != null && otherGlyphs.size() > 0) {
            g = (Glyph) otherGlyphs.firstElement();
            if (g.getOwner() != null) {
                getAndDisplayURL((LElem) g.getOwner(), g);
            }
        }
    }

    protected void getAndDisplayURL(final LElem noa, final Glyph g) {
        final String url = noa.getURL(g);
        if (url != null && url.length() > 0) {
            this.application.displayURLinBrowser(url);
        }
    }

}
